const RootTypes = `
    # Requêtes
    type Query {
        allGroups: [Group]
        group(uid: ID) : Group
        user(uid: ID) : [User]
        allEvents: [Event]

        asAdmin(groupUID: ID): AdminQuery

        allPosts: [Post]
        
        searchTOL(
            givenName: String,
            lastName: String,
            nickname: String,
            nationality: String,
            school: String,
            promotion: String,
            groups: String,
            studies: String,
            sport: String,
            phone: String,
            mail: String,
            adress: String,
            ip: String
        ): [String]
    }

    type Mutation {
        asAdmin(groupUID: String): AdminMutation
        asSpeaker(groupUID: String): SpeakerMutation
        asMember(groupUID: String): MemberMutation
        asViewer(groupUID: String): ViewerMutation
    }

   
`;

const subMutations = `
    type AdminMutation {

        isAdmin: Boolean

        createSubgroup(
            uid: ID = null,
            name: String,
            website: String,
            description: String,
            school: String
        ): Group

        addUser(userid : String): User
        removeUser(userid : String): User
        addAdmin(userid : String): User
        removeAdmin(userid : String): User

        editGroup(
            name: String,
            website: String,
            description: String,
            school: String
        ): Group

    }

    type SpeakerMutation{
        postEvent(name: String, date: String): Event
    }

    type MemberMutation {
        leave: Group
    }

    type ViewerMutation {
        requestJoin: Group
    }

`;

const subQueries = `
    type AdminQuery{
        isAdmin: Boolean
        allRequests : Requests
    }
`;

const User = `
    # Utilisateurs
    type User {
        # Prénom de l'utilisateur
        givenName: String!
        # Nom de famille
        lastName: String!
        uid: ID!
        birthdate: String!
        mail: String
        phone: String
        # Groupes dont l'utilisateur est membre.
        groups: [Group]
        # Groupes que l'utilisateur aime.
        likes: [Group]
        address: String
    }
`;

const Group = `
    # Groupes associatifs
    type Group {
        uid: ID
        name: String
        website: String
        createdAt: String!
        updatedAt: String!
        description: String
        school: String
        parent: Group
    }
`;

const Post = `
    # Publications
    interface Post {
        id: ID!
        title: String!
        createdAt: String!
        updatedAt: String!
        # Auteurs de la publication
        author: [Group]
        description: String
    }
`;

const Events = `
    # Événements
    type Event implements Post {
        id: ID!
        # Intitulé de l'événement
        title: String!
        # Lieu de l'événement
        location: String
        createdAt: String!
        updatedAt: String!
        startTime: String!
        endTime: String!
        # Organisateurs
        author: [Group]
        # Personnes qui y participent
        participants: [User]
        description: String
    }
`;

const Requests = `
    type Requests {
        userJoinGroup: [UserJoinGroup]
        groupJoinEvent: [GroupJoinEvent]
        yourGroupHostEvent: [YourGroupHostEvent]
    }

    type UserJoinGroup {
        requestUID: ID
        user: User
        message: String
    }

    type GroupJoinEvent {
        event: Event
        groupWantingToJoin: Group
    }

    # Demande au récipiendaire de rejoindre l'organisation d'un événement.
    type YourGroupHostEvent {
        event: Event
        sender: Group
    }

`;

const typeDefs = [RootTypes, Group, User, Post, Events, subMutations, subQueries, Requests];

export default typeDefs;